@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2010 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -30,6 +30,7 @@
 #include "misc.h"
 #include "event.h"
 #include "socket.h"
+#include "mroute.h"
 
 #define MANAGEMENT_VERSION                      1
 #define MANAGEMENT_N_PASSWORD_RETRIES           3
@@ -37,6 +38,24 @@
 #define MANAGEMENT_ECHO_BUFFER_SIZE           100
 #define MANAGEMENT_STATE_BUFFER_SIZE          100
 
+/*
+ * Management-interface-based deferred authentication
+ */
+#ifdef MANAGEMENT_DEF_AUTH
+struct man_def_auth_context {
+  unsigned long cid;
+
+#define DAF_CONNECTION_ESTABLISHED (1<<0)
+#define DAF_CONNECTION_CLOSED      (1<<1)
+#define DAF_INITIAL_AUTH           (1<<2)
+  unsigned int flags;
+
+  unsigned int mda_key_id_counter;
+
+  time_t bytecount_last_update;
+};
+#endif
+
 /*
  * Manage build-up of command line
  */
@@ -54,34 +73,6 @@ const unsigned char *command_line_get (struct command_line *cl);
 void command_line_reset (struct command_line *cl);
 void command_line_next (struct command_line *cl);
 
-/*
- * Manage lists of output strings
- */
-
-struct output_entry
-{
-  struct buffer buf;
-  struct output_entry *next;
-};
-
-struct output_list
-{
-  struct output_entry *head; /* next item to pop/peek */
-  struct output_entry *tail; /* last item pushed */
-  int size;                  /* current number of entries */
-  int max_size;              /* maximum size list should grow to */
-};
-
-struct output_list *output_list_new (const int max_size);
-void output_list_free (struct output_list *ol);
-
-bool output_list_defined (const struct output_list *ol);
-void output_list_reset (struct output_list *ol);
-
-void output_list_push (struct output_list *ol, const unsigned char *str);
-const struct buffer *output_list_peek (struct output_list *ol);
-void output_list_advance (struct output_list *ol, int n);
-
 /*
  * Manage log file history
  */
@@ -89,6 +80,7 @@ void output_list_advance (struct output_list *ol, int n);
 union log_entry_union {
   unsigned int msg_flags;
   int state;
+  int intval;
 };
 
 struct log_entry
@@ -96,6 +88,7 @@ struct log_entry
   time_t timestamp;
   const char *string;
   in_addr_t local_ip;
+  in_addr_t remote_ip;
   union log_entry_union u;
 };
 
@@ -111,6 +104,12 @@ struct log_entry
 #define LOG_PRINT_CRLF         (1<<7)
 #define LOG_FATAL_NOTIFY       (1<<8)
 
+#define LOG_PRINT_INTVAL       (1<<9)
+
+#define LOG_PRINT_REMOTE_IP    (1<<10)
+
+#define LOG_ECHO_TO_LOG        (1<<11)
+
 const char *log_entry_print (const struct log_entry *e, unsigned int flags, struct gc_arena *gc);
 
 struct log_history
@@ -140,16 +139,41 @@ log_history_capacity (const struct log_history *h)
 }
 
 /*
- * Callbacks for 'status' and 'kill' commands
+ * Callbacks for 'status' and 'kill' commands.
+ * Also for management-based deferred authentication and packet filter.
  */
 struct management_callback
 {
   void *arg;
+
+# define MCF_SERVER (1<<0) /* is OpenVPN being run as a server? */
+  unsigned int flags;
+
   void (*status) (void *arg, const int version, struct status_output *so);
   void (*show_net) (void *arg, const int msglevel);
   int (*kill_by_cn) (void *arg, const char *common_name);
   int (*kill_by_addr) (void *arg, const in_addr_t addr, const int port);
   void (*delete_event) (void *arg, event_t event);
+  int (*n_clients) (void *arg);
+#ifdef MANAGEMENT_DEF_AUTH
+  bool (*kill_by_cid) (void *arg, const unsigned long cid);
+  bool (*client_auth) (void *arg,
+		       const unsigned long cid,
+		       const unsigned int mda_key_id,
+		       const bool auth,
+		       const char *reason,
+		       const char *client_reason,
+		       struct buffer_list *cc_config); /* ownership transferred */
+  char *(*get_peer_info) (void *arg, const unsigned long cid);
+#endif
+#ifdef MANAGEMENT_PF
+  bool (*client_pf) (void *arg,
+		     const unsigned long cid,
+		     struct buffer_list *pf_config);   /* ownership transferred */
+#endif
+#if HTTP_PROXY_FALLBACK
+  bool (*http_proxy_fallback_cmd) (void *arg, const char *server, const char *port, const char *flags);
+#endif
 };
 
 /*
@@ -181,25 +205,40 @@ struct man_persist {
   bool hold_release;
 
   const char *special_state_msg;
+
+  counter_type bytes_in;
+  counter_type bytes_out;
 };
 
 struct man_settings {
   bool defined;
-  struct sockaddr_in local;
-  bool up_query_passwords;
+  unsigned int flags; /* MF_x flags */
+  struct openvpn_sockaddr local;
+#if UNIX_SOCK_SUPPORT
+  struct sockaddr_un local_unix;
+#endif
   bool management_over_tunnel;
   struct user_pass up;
   int log_history_cache;
   int echo_buffer_size;
   int state_buffer_size;
-  bool server;
-  bool hold;
+  char *write_peer_info_file;
+  int client_uid;
+  int client_gid;
+
+/* flags for handling the management interface "signal" command */
+# define MANSIG_IGNORE_USR1_HUP  (1<<0)
+# define MANSIG_MAP_USR1_TO_HUP  (1<<1)
+# define MANSIG_MAP_USR1_TO_TERM (1<<2)
+  unsigned int mansig;
 };
 
 /* up_query modes */
 #define UP_QUERY_DISABLED  0
 #define UP_QUERY_USER_PASS 1
 #define UP_QUERY_PASS      2
+#define UP_QUERY_NEED_OK   3
+#define UP_QUERY_NEED_STR  4
 
 /* states */
 #define MS_INITIAL          0  /* all sockets are closed */
@@ -212,7 +251,7 @@ struct man_connection {
 
   socket_descriptor_t sd_top;
   socket_descriptor_t sd_cli;
-  struct sockaddr_in remote;
+  struct openvpn_sockaddr remote;
 
 #ifdef WIN32
   struct net_event_win32 ne32;
@@ -223,13 +262,25 @@ struct man_connection {
   int password_tries;
 
   struct command_line *in;
-  struct output_list *out;
+  struct buffer_list *out;
 
+#ifdef MANAGEMENT_DEF_AUTH
+# define IEC_UNDEF       0
+# define IEC_CLIENT_AUTH 1
+# define IEC_CLIENT_PF   2
+  int in_extra_cmd;
+  unsigned long in_extra_cid;
+  unsigned int in_extra_kid;
+  struct buffer_list *in_extra;
+  int env_filter_level;
+#endif
   struct event_set *es;
 
   bool state_realtime;
   bool log_realtime;
   bool echo_realtime;
+  int bytecount_update_seconds;
+  time_t bytecount_last_update;
 
   const char *up_query_type;
   int up_query_mode;
@@ -249,17 +300,33 @@ struct user_pass;
 
 struct management *management_init (void);
 
+/* management_open flags */
+# define MF_SERVER            (1<<0)
+# define MF_QUERY_PASSWORDS   (1<<1)
+# define MF_HOLD              (1<<2)
+# define MF_SIGNAL            (1<<3)
+# define MF_FORGET_DISCONNECT (1<<4)
+# define MF_CONNECT_AS_CLIENT (1<<5)
+#ifdef MANAGEMENT_DEF_AUTH
+# define MF_CLIENT_AUTH       (1<<6)
+#endif
+#ifdef MANAGEMENT_PF
+# define MF_CLIENT_PF         (1<<7)
+#endif
+# define MF_UNIX_SOCK       (1<<8)
+
 bool management_open (struct management *man,
 		      const char *addr,
 		      const int port,
 		      const char *pass_file,
-		      const bool server,
-		      const bool query_passwords,
+		      const char *client_user,
+		      const char *client_group,
 		      const int log_history_cache,
 		      const int echo_buffer_size,
 		      const int state_buffer_size,
-		      const bool hold);
-
+		      const char *write_peer_info_file,
+		      const int remap_sigusr1,
+		      const unsigned int flags);
 
 void management_close (struct management *man);
 
@@ -279,7 +346,7 @@ void management_set_callback (struct management *man,
 
 void management_clear_callback (struct management *man);
 
-bool management_query_user_pass (struct management *man, struct user_pass *up, const char *type, const bool password_only);
+bool management_query_user_pass (struct management *man, struct user_pass *up, const char *type, const unsigned int flags);
 
 bool management_should_daemonize (struct management *man);
 bool management_would_hold (struct management *man);
@@ -287,6 +354,26 @@ bool management_hold (struct management *man);
 
 void management_event_loop_n_seconds (struct management *man, int sec);
 
+#ifdef MANAGEMENT_DEF_AUTH
+void management_notify_client_needing_auth (struct management *management,
+					    const unsigned int auth_id,
+					    struct man_def_auth_context *mdac,
+					    const struct env_set *es);
+
+void management_connection_established (struct management *management,
+					struct man_def_auth_context *mdac,
+					const struct env_set *es);
+
+void management_notify_client_close (struct management *management,
+				     struct man_def_auth_context *mdac,
+				     const struct env_set *es);
+
+void management_learn_addr (struct management *management,
+			    struct man_def_auth_context *mdac,
+			    const struct mroute_addr *addr,
+			    const bool primary);
+#endif
+
 static inline bool
 management_connected (const struct management *man)
 {
@@ -296,9 +383,25 @@ management_connected (const struct management *man)
 static inline bool
 management_query_user_pass_enabled (const struct management *man)
 {
-  return man->settings.up_query_passwords;
+  return BOOL_CAST(man->settings.flags & MF_QUERY_PASSWORDS);
 }
 
+#ifdef MANAGEMENT_PF
+static inline bool
+management_enable_pf (const struct management *man)
+{
+  return man && BOOL_CAST(man->settings.flags & MF_CLIENT_PF);
+}
+#endif
+
+#ifdef MANAGEMENT_DEF_AUTH
+static inline bool
+management_enable_def_auth (const struct management *man)
+{
+  return man && BOOL_CAST(man->settings.flags & MF_CLIENT_AUTH);
+}
+#endif
+
 /*
  * OpenVPN tells the management layer what state it's in
  */
@@ -316,26 +419,98 @@ management_query_user_pass_enabled (const struct management *man)
 #define OPENVPN_STATE_WAIT          7  /* Waiting for initial response from server */
 #define OPENVPN_STATE_AUTH          8  /* Authenticating with server */
 #define OPENVPN_STATE_GET_CONFIG    9  /* Downloading configuration from server */
+#define OPENVPN_STATE_RESOLVE       10 /* DNS lookup */
+#define OPENVPN_STATE_TCP_CONNECT   11 /* Connecting to TCP server */
 
 #define OPENVPN_STATE_CLIENT_BASE   7  /* Base index of client-only states */
 
 void management_set_state (struct management *man,
 			   const int state,
 			   const char *detail,
-			   const in_addr_t tun_local_ip);
+			   const in_addr_t tun_local_ip,
+			   const in_addr_t tun_remote_ip);
 
 /*
  * The management object keeps track of OpenVPN --echo
  * parameters.
  */
-void management_echo (struct management *man, const char *string);
+void management_echo (struct management *man, const char *string, const bool pull);
 
 /*
  * OpenVPN calls here to indicate a password failure
  */
 
-void management_auth_failure (struct management *man, const char *type);
+void management_auth_failure (struct management *man, const char *type, const char *reason);
+
+/*
+ * These functions drive the bytecount in/out counters.
+ */
+
+void man_bytecount_output_client (struct management *man);
+
+static inline void
+man_bytecount_possible_output_client (struct management *man)
+{
+  if (man->connection.bytecount_update_seconds > 0
+      && now >= man->connection.bytecount_last_update
+      + man->connection.bytecount_update_seconds)
+    man_bytecount_output_client (man);
+}
+
+static inline void
+management_bytes_out_client (struct management *man, const int size)
+{
+  man->persist.bytes_out += size;
+  man_bytecount_possible_output_client (man);
+}
+
+static inline void
+management_bytes_in_client (struct management *man, const int size)
+{
+  man->persist.bytes_in += size;
+  man_bytecount_possible_output_client (man);
+}
+
+static inline void
+management_bytes_out (struct management *man, const int size)
+{
+  if (!(man->persist.callback.flags & MCF_SERVER))
+    management_bytes_out_client (man, size);
+}
+
+static inline void
+management_bytes_in (struct management *man, const int size)
+{
+  if (!(man->persist.callback.flags & MCF_SERVER))
+    management_bytes_in_client (man, size);
+}
+
+#ifdef MANAGEMENT_DEF_AUTH
+
+static inline void
+management_bytes_server (struct management *man,
+			 const counter_type *bytes_in_total,
+			 const counter_type *bytes_out_total,
+			 struct man_def_auth_context *mdac)
+{
+  void man_bytecount_output_server (struct management *man,
+				    const counter_type *bytes_in_total,
+				    const counter_type *bytes_out_total,
+				    struct man_def_auth_context *mdac);
+
+  if (man->connection.bytecount_update_seconds > 0
+      && now >= mdac->bytecount_last_update + man->connection.bytecount_update_seconds
+      && (mdac->flags & (DAF_CONNECTION_ESTABLISHED|DAF_CONNECTION_CLOSED)) == DAF_CONNECTION_ESTABLISHED)
+    man_bytecount_output_server (man, bytes_in_total, bytes_out_total, mdac);
+}
+
+#endif /* MANAGEMENT_DEF_AUTH */
+
+#if HTTP_PROXY_FALLBACK
+
+void management_http_proxy_fallback_notify (struct management *man, const char *type, const char *remote_ip_hint);
+
+#endif /* HTTP_PROXY_FALLBACK */
 
 #endif
-
 #endif
